/*
 *  Copyright (C) 2012 Simon Robinson
 * 
 *  This file is part of Com-Me.
 * 
 *  Com-Me is free software; you can redistribute it and/or modify it 
 *  under the terms of the GNU Lesser General Public License as 
 *  published by the Free Software Foundation; either version 3 of the 
 *  License, or (at your option) any later version.
 *
 *  Com-Me is distributed in the hope that it will be useful, but WITHOUT 
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General 
 *  Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with Com-Me.
 *  If not, see <http://www.gnu.org/licenses/>.
 */

package ac.robinson.util;

import java.util.Arrays;

public class AndroidUtilities {

	public static final String APPLICATION_NAME = "androidutilities";
	public static final String LOG_TAG = APPLICATION_NAME;

	/**
	 * Merge two arrays. BEWARE: some versions of Android (<2.2?) have a buggy System.arraycopy, so this could silently
	 * fail, particularly on larger arrays
	 * 
	 * @param first
	 * @param second
	 * @return
	 */
	public static <T> T[] concatenateArrays(T[] first, T[] second) {
		T[] result = Arrays.copyOf(first, first.length + second.length);
		System.arraycopy(second, 0, result, first.length, second.length);
		return result;
	}

	/**
	 * Merge two arrays. BEWARE: some versions of Android (<2.2?) have a buggy System.arraycopy, so this could silently
	 * fail, particularly on larger arrays
	 * 
	 * @param first
	 * @param second
	 * @return
	 */
	public static int[] concatenateArrays(int[] first, int[] second) {
		int[] result = Arrays.copyOf(first, first.length + second.length);
		System.arraycopy(second, 0, result, first.length, second.length);
		return result;
	}
}
